/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2018-2023. All rights reserved.
 */

package com.huawei.apigateway.backendsignature.filters;

import com.cloud.apigateway.sdk.utils.Request;
import com.cloud.sdk.auth.signer.Signer;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.TimeZone;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class ApigatewaySignatureFilter implements Filter {
    private static Map<String, String> secrets = new HashMap<>();
    private static final Pattern authorizationPattern = Pattern
            .compile("SDK-HMAC-SHA256\\s+Access=([^,]+),\\s?SignedHeaders=([^,]+),\\s?Signature=(\\w+)");
    public static final SimpleDateFormat TIME_FORMATTER = new SimpleDateFormat("yyyyMMdd'T'HHmmss'Z'");
    private static final Logger LOGGER = LoggerFactory.getLogger(ApigatewaySignatureFilter.class);

    static {
        // 认证用的ak和sk硬编码到代码中或者明文存储都有很大的安全风险，建议在配置文件或者环境变量中密文存放，使用时解密，确保安全；
        // 本示例以ak和sk保存在环境变量中为例，运行本示例前请先在本地环境中设置环境变量HUAWEICLOUD_SDK_AK1和HUAWEICLOUD_SDK_SK1, HUAWEICLOUD_SDK_AK2和HUAWEICLOUD_SDK_SK2。
        secrets.put(System.getenv("HUAWEICLOUD_SDK_AK1"), System.getenv("HUAWEICLOUD_SDK_SK1"));
        secrets.put(System.getenv("HUAWEICLOUD_SDK_AK2"), System.getenv("HUAWEICLOUD_SDK_SK2"));
        TIME_FORMATTER.setTimeZone(TimeZone.getTimeZone("UTC"));
    }

    @Override
    public void doFilter(ServletRequest servletRequest, ServletResponse servletResponse, FilterChain chain) {
        HttpServletResponse response = (HttpServletResponse) servletResponse;
        try {
            RequestWrapper request = new RequestWrapper((HttpServletRequest) servletRequest);

            String authorization = request.getHeader("Authorization");
            if (authorization == null || authorization.length() == 0) {
                response.sendError(HttpServletResponse.SC_UNAUTHORIZED, "Authorization not found.");
                return;
            }

            Matcher m = authorizationPattern.matcher(authorization);
            if (!m.find()) {
                response.sendError(HttpServletResponse.SC_UNAUTHORIZED, "Authorization format incorrect.");
                return;
            }
            String signingKey = m.group(1);
            String signingSecret = secrets.get(signingKey);
            if (signingSecret == null) {
                response.sendError(HttpServletResponse.SC_UNAUTHORIZED, "Signing key not found.");
                return;
            }
            String[] signedHeaders = m.group(2).split(";");

            Request apiRequest = new Request();
            apiRequest.setMethod(request.getMethod());
            String url = request.getRequestURL().toString();
            String queryString = request.getQueryString();
            if (null != queryString && !"".equals(queryString)) {
                url = url + "?" + queryString;
            }
            apiRequest.setUrl(url);

            boolean needbody = true;
            String dateHeader = null;
            for (int i = 0; i < signedHeaders.length; i++) {
                String headerValue = request.getHeader(signedHeaders[i]);
                if (headerValue == null || headerValue.length() == 0) {
                    response.sendError(HttpServletResponse.SC_UNAUTHORIZED,
                            "Signed header" + signedHeaders[i] + " not found.");
                } else {
                    apiRequest.addHeader(signedHeaders[i], headerValue);// set header
                    if (signedHeaders[i].equalsIgnoreCase("x-sdk-content-sha256")
                            && headerValue.equals("UNSIGNED-PAYLOAD")) {
                        needbody = false;
                    }
                    if (signedHeaders[i].equalsIgnoreCase("x-sdk-date")) {
                        dateHeader = headerValue;
                    }
                }
            }
            if (dateHeader == null) {
                response.sendError(HttpServletResponse.SC_UNAUTHORIZED, "Header x-sdk-date not found.");
                return;
            }
            Date date = TIME_FORMATTER.parse(dateHeader);
            long duration = Math.abs(new Date().getTime() - date.getTime());
            if (duration > 15 * 60 * 1000) {
                response.sendError(HttpServletResponse.SC_UNAUTHORIZED, "Signature expired.");
                return;
            }

            if (needbody) {
                apiRequest.setBody(new String(request.getBody(), StandardCharsets.UTF_8)); // set body
            } else {
                apiRequest.setBody("");
            }

            apiRequest.addHeader("authorization", authorization);
            apiRequest.setKey(signingKey);
            apiRequest.setSecret(signingSecret);
            Signer signer = new Signer();
            boolean verify = signer.verify(apiRequest);
            if (verify) {
                chain.doFilter(request, response);
            } else {
                response.sendError(HttpServletResponse.SC_UNAUTHORIZED, "Verify authroization failed.");
            }
        } catch (Exception e) {
            LOGGER.error(e.getMessage());
            try {
                response.sendError(HttpServletResponse.SC_INTERNAL_SERVER_ERROR);
            } catch (IOException e1) {
            }
        }
    }

}